@@ -26,15 +26,11 @@ _absent = object()
 # between a URL Path and a Fragment Path and a URL Query and a Fragment Query.
 #
 # For example, '?' and '#' don't need to be encoded in Fragment Path segments
-# but they must be encoded in URL Path segments.
+# but must be encoded in URL Path segments.
 #
 # Similarly, '#' doesn't need to be encoded in Fragment Query keys and values,
 # but must be encoded in URL Query keys and values.
 #
-# Also, force_absolute_if_not_empty is True in URL Paths but False in Fragment
-# Paths. This is already handled, but its implementation can change once the new
-# classes are implemented.
-#
 
 class Path(object):
   """
@@ -42,16 +38,16 @@ class Path(object):
 
     http://tools.ietf.org/html/rfc3986#section-3.3
 
-  Path parameters are currently not supported.
+  Path parameters aren't supported.
 
   Attributes:
-    _absolute_if_not_empty: Boolean whether or not isabsolute should be forced
-      to True if the path is non-empty. If true, isabsolute is read
-      only. _absolute_if_not_empty is True for URL paths, False for Fragment
-      paths. URL paths are always absolute (isabsolute is True) if they are not
-      empty, and thus the user shouldn't be able to set isabsolute, only read
-      it. Fragment paths can be optionally absolute, and thus it makes sense to
-      allow the user to set isabsolute to True or False.
+    _force_absolute: Function whos boolean return value specifies whether
+      self.isabsolute should be forced to True or not. If _force_absolute(self)
+      returns True, isabsolute is read only and raises an AttributeError if
+      assigned to. If _force_absolute(self) returns False, isabsolute is mutable
+      and can be set to True or False. URL paths use _force_absolute and return
+      True if the netloc is non-empty (not equal to ''). Fragment paths are
+      never read-only and their _force_absolute(self) always returns False.
     segments: List of zero or more path segments comprising this path. If the
       path string has a trailing '/', the last segment will be '' and self.isdir
       will be True and self.isfile will be False. An empty segment list
@@ -65,12 +61,12 @@ class Path(object):
   """
   SAFE_SEGMENT_CHARS = ":@-._~!$&'()*+,;="
   
-  def __init__(self, path='', absolute_if_not_empty=False, strict=False):
+  def __init__(self, path='', force_absolute=lambda _: False, strict=False):
     self.segments = []
 
     self.strict = strict
     self._isabsolute = False
-    self._absolute_if_not_empty = absolute_if_not_empty
+    self._force_absolute = force_absolute
 
     self.load(path)
 
@@ -88,7 +84,7 @@ class Path(object):
     else: # List interface.
       segments = path
 
-    if self._absolute_if_not_empty:
+    if self._force_absolute(self):
       self._isabsolute = True if segments else False
     else:
       self._isabsolute = (segments and segments[0] == '')
@@ -134,19 +130,20 @@ class Path(object):
 
   @property
   def isabsolute(self):
-    if self._absolute_if_not_empty and self.segments:
+    if self._force_absolute(self):
       return True
     return self._isabsolute
 
   @isabsolute.setter
   def isabsolute(self, isabsolute):
     """
-    Raises: AttributeError if _absolute_if_not_empty is True.
+    Raises: AttributeError if _force_absolute(self) returns True.
     """
-    if self._absolute_if_not_empty:
-      errstr = ('Path.isabsolute is read only for URL paths.'
-                'URL paths are always absolute if not empty.')
-      raise AttributeError(errstr)
+    if self._force_absolute(self):
+      s = ('Path.isabsolute is True and read-only for URLs with a netloc (a '
+           'username, password, host, and/or port). A URL path must start with '
+           "a '/' to separate itself from a netloc.")
+      raise AttributeError(s)
     self._isabsolute = isabsolute
 
   @property
@@ -170,8 +167,11 @@ class Path(object):
 
   def __str__(self):
     segments = list(self.segments)
-    if self.isabsolute and self.segments:
-      segments.insert(0, '')
+    if self.isabsolute:
+      if not segments:
+        segments = ['', '']
+      else:
+        segments.insert(0, '')
     return self._path_from_segments(segments, quoted=True)
     
   def __repr__(self):
@@ -219,13 +219,14 @@ class PathCompositionInterface(object):
   Abstract class interface for a parent class that contains a Path.
   """
   __metaclass__ = abc.ABCMeta
-  def __init__(self, absolute_if_not_empty=False, strict=False):
+  def __init__(self, strict=False):
     """
     Params:
-      absolute_if_not_empty: See Path._absolute_if_not_empty.
+      force_absolute: See Path._force_absolute.
+
+    Assignments to <self> in __init__() must be added to __setattr__() below.
     """
-    self._path = Path(absolute_if_not_empty=absolute_if_not_empty,
-                      strict=strict)
+    self._path = Path(force_absolute=self._force_absolute, strict=strict)
 
   @property
   def path(self):
@@ -233,18 +234,71 @@ class PathCompositionInterface(object):
   
   @property
   def pathstr(self):
-    """This method will be deprecated in favor of str(f.path) in furl v0.3.5."""
+    """This method is deprecated. Use str(furl.path) instead."""
+    s = ('furl.pathstr is deprecated. Use str(furl.path) instead. There '
+         'should be one, and preferably only one, obvious way to serialize '
+         'a Path object to a string.')
+    warnings.warn(s, DeprecationWarning)
     return str(self._path)
 
   def __setattr__(self, attr, value):
     """
     Returns: True if this attribute is handled and set here, False otherwise.
     """
-    if attr == 'path':
+    if attr == '_path':
+      self.__dict__[attr] = value
+      return True
+    elif attr == 'path':
       self._path.load(value)
       return True
     return False
 
+  @abc.abstractmethod
+  def _force_absolute(self, path):
+    """
+    Subclass me.
+    """
+    pass
+
+
+class URLPathCompositionInterface(PathCompositionInterface):
+  """
+  Abstract class interface for a parent class that contains a URL Path.
+
+  A URL path's isabsolute attribute is absolute and read-only if a netloc is
+  defined. A path cannot start without '/' if there's a netloc. For example, the
+  URL 'http://google.coma/path' makes no sense. It should be
+  'http://google.com/a/path'.
+
+  A URL path's isabsolute attribute is mutable if there's no netloc. The scheme
+  doesn't matter. For example, the isabsolute attribute of the URL path in
+  'mailto:user@domain.com', with scheme 'mailto' and path 'user@domain.com', is
+  mutable because there is no netloc. See
+
+    http://en.wikipedia.org/wiki/URI_scheme#Examples
+  """
+  __metaclass__ = abc.ABCMeta
+  def __init__(self, strict=False):
+    PathCompositionInterface.__init__(self, strict=strict)
+
+  def _force_absolute(self, path):
+    return bool(path) and self.netloc
+
+
+class FragmentPathCompositionInterface(PathCompositionInterface):
+  """
+  Abstract class interface for a parent class that contains a Fragment Path.
+
+  Fragment Paths they be set to absolute (self.isabsolute = True) or not
+  absolute (self.isabsolute = False).
+  """
+  __metaclass__ = abc.ABCMeta
+  def __init__(self, strict=False):
+    PathCompositionInterface.__init__(self, strict=strict)
+
+  def _force_absolute(self, path):
+    return False
+
 
 class Query(object):
   """
@@ -463,9 +517,11 @@ class QueryCompositionInterface(object):
 
   @property
   def querystr(self):
-    """
-    This method will be deprecated in favor of str(f.query) in furl v0.3.5.
-    """
+    """This method is deprecated. Use str(furl.query) instead."""
+    s = ('furl.querystr is deprecated. Use str(furl.query) instead. There '
+         'should be one, and preferably only one, obvious way to serialize '
+         'a Query object to a string.')
+    warnings.warn(s, DeprecationWarning)
     return str(self._query)
 
   @property
@@ -485,7 +541,7 @@ class QueryCompositionInterface(object):
     return False
 
 
-class Fragment(PathCompositionInterface, QueryCompositionInterface):
+class Fragment(FragmentPathCompositionInterface, QueryCompositionInterface):
   """
   Represents a URL fragment, comprised internally of a Path and Query optionally
   separated by a '?' character.
@@ -493,8 +549,8 @@ class Fragment(PathCompositionInterface, QueryCompositionInterface):
     http://tools.ietf.org/html/rfc3986#section-3.5
 
   Attributes:
-    path: Path object representing the path portion of this fragment.
-    query: Query object representing the query portion of this fragment.
+    path: Path object from FragmentPathCompositionInterface.
+    query: Query object from QueryCompositionInterface.
     separator: Boolean whether or not a '?' separator should be included in the
       string representation of this fragment. When False, a '?' character will
       not separate the fragment path from the fragment query in the fragment
@@ -502,8 +558,7 @@ class Fragment(PathCompositionInterface, QueryCompositionInterface):
       where no separating '?' is desired.
   """
   def __init__(self, fragment='', strict=False):
-    PathCompositionInterface.__init__(self, absolute_if_not_empty=False,
-                                      strict=strict)
+    FragmentPathCompositionInterface.__init__(self, strict=strict)
     QueryCompositionInterface.__init__(self, strict=strict)
     self.strict = strict
     self.separator = True
@@ -599,9 +654,11 @@ class FragmentCompositionInterface(object):
 
   @property
   def fragmentstr(self):
-    """
-    This method will be deprecated in favor of str(f.fragmentstr) in furl v0.3.5.
-    """
+    """This method is deprecated. Use str(furl.fragment) instead."""
+    s = ('furl.fragmentstr is deprecated. Use str(furl.fragment) instead. There'
+         ' should be one, and preferably only one, obvious way to serialize '
+         'a Fragment object to a string.')
+    warnings.warn(s, DeprecationWarning)
     return str(self._fragment)
   
   def __setattr__(self, attr, value):
@@ -614,7 +671,7 @@ class FragmentCompositionInterface(object):
     return False
 
 
-class furl(PathCompositionInterface, QueryCompositionInterface,
+class furl(URLPathCompositionInterface, QueryCompositionInterface,
            FragmentCompositionInterface):
   """
   Object for simple parsing and manipulation of a URL and its components.
@@ -636,7 +693,7 @@ class furl(PathCompositionInterface, QueryCompositionInterface,
     port: Port. Valid port values are 1-65535, or None meaning no port
       specified.
     netloc: Network location. Combined host and port string. Initially None.
-    path: Path object from PathCompositionInterface.
+    path: Path object from URLPathCompositionInterface.
     query: Query object from QueryCompositionInterface.
     fragment: Fragment object from FragmentCompositionInterface.
   """
@@ -651,8 +708,7 @@ class furl(PathCompositionInterface, QueryCompositionInterface,
     """
     Raises: ValueError on invalid url.
     """
-    PathCompositionInterface.__init__(self, absolute_if_not_empty=True,
-                                      strict=strict)
+    URLPathCompositionInterface.__init__(self, strict=strict)
     QueryCompositionInterface.__init__(self, strict=strict)
     FragmentCompositionInterface.__init__(self, strict=strict)
     self.strict = strict
@@ -983,7 +1039,7 @@ class furl(PathCompositionInterface, QueryCompositionInterface,
     return self
 
   def join(self, url):
-    self.load(urljoin(self.url, url))
+    self.load(urljoin(self.url, str(url)))
     return self
 
   def copy(self):
@@ -999,15 +1055,17 @@ class furl(PathCompositionInterface, QueryCompositionInterface,
       object.__setattr__(self, attr, value)
 
   def __str__(self):
-    tokens = (self.scheme, self.netloc, self.pathstr, self.querystr,
-              self.fragmentstr)
-    url = urlparse.urlunsplit(tokens)
-    if not self.scheme and url.startswith('//'): # No scheme.
+    path, query, fragment = str(self.path), str(self.query), str(self.fragment)
+    url = urlparse.urlunsplit((self.scheme, self.netloc, path, query, fragment))
+
+    # Special cases.
+    if not self.scheme and url.startswith('//') and not path.startswith('//'):
       url = url[2:]
-    elif self.scheme is not None and url == '': # Scheme only.
+    elif self.scheme is not None and url == '':
       url += '://'
-    elif self.scheme is not None and url == '%s:' % self.scheme: # Scheme only.
+    elif self.scheme is not None and url == '%s:' % self.scheme:
       url += '//'
+
     return url
 
   def __repr__(self):
