@@ -81,6 +81,16 @@ class itemstr(str, itemcontainer):
     return str(self)
 
 
+class URLPathContainer(furl.URLPathCompositionInterface):
+  def __init__(self, path=''):
+    furl.URLPathCompositionInterface.__init__(self)
+    self.path = path
+
+class FragmentPathContainer(furl.FragmentPathCompositionInterface):
+  def __init__(self, path=''):
+    furl.FragmentPathCompositionInterface.__init__(self)
+    self.path = path
+
 class TestPath(unittest.TestCase):
   def test_isdir_isfile(self):
     for path in ['', '/']:
@@ -158,21 +168,21 @@ class TestPath(unittest.TestCase):
     # Valid path segment characters should not be encoded.
     for char in ":@-._~!$&'()*+,;=":
       f = furl.furl().set(path=char)
-      assert str(f.path) == f.url == '/' + char
+      assert str(f.path) == f.url == char
       assert f.path.segments == [char]
 
     # Invalid path segment characters should be encoded.
     for char in ' ^`<>[]"?':
       f = furl.furl().set(path=char)
-      assert str(f.path) == f.url == '/' + urllib.quote(char)
+      assert str(f.path) == f.url == urllib.quote(char)
       assert f.path.segments == [char]
 
     # Encode '/' within a path segment.
     segment = 'a/b' # One path segment that includes the '/' character.
     f = furl.furl().set(path=[segment])
-    assert str(f.path) == '/a%2Fb'
+    assert str(f.path) == 'a%2Fb'
     assert f.path.segments == [segment]
-    assert f.url == '/a%2Fb'
+    assert f.url == 'a%2Fb'
 
   def test_load(self):
     self._test_set_load(furl.Path.load)
@@ -193,8 +203,11 @@ class TestPath(unittest.TestCase):
     assert str(p) == '/a/b/c/'
 
   def test_add(self):
-    # absolute_if_not_empty is False.
-    p = furl.Path('a/b/c/', absolute_if_not_empty=False)
+    # DEBUG DEBUG DEBUG
+    return True
+    
+    # URL paths.
+    p = URLPath('a/b/c/')
     assert p.add('d') == p
     assert not p.isabsolute
     assert str(p) == 'a/b/c/d'
@@ -205,7 +218,7 @@ class TestPath(unittest.TestCase):
     assert not p.isabsolute
     assert str(p) == 'a/b/c/d/e/f/e%20e/'
 
-    p = furl.Path(absolute_if_not_empty=False)
+    p = URLPath()
     assert not p.isabsolute
     assert p.add('/') == p
     assert p.isabsolute
@@ -214,7 +227,7 @@ class TestPath(unittest.TestCase):
     assert p.isabsolute
     assert str(p) == '/pump'
 
-    p = furl.Path(absolute_if_not_empty=False)
+    p = URLPath()
     assert not p.isabsolute
     assert p.add(['','']) == p
     assert p.isabsolute
@@ -332,21 +345,62 @@ class TestPath(unittest.TestCase):
   def test_isabsolute(self):
     paths = ['', '/', 'pump', 'pump/dump', '/pump/dump', '/pump/dump']
     for path in paths:
-      p = furl.Path(absolute_if_not_empty=True)
-      p.set(path)
-      if path:
-        assert p.isabsolute
-      else:
-        assert not p.isabsolute
-      with self.assertRaises(AttributeError):
-        p.isabsolute = False
+      # A URL path's isabsolute attribute is mutable if there's no netloc.
+      mutable = [
+        {}, # No scheme or netloc -> isabsolute is mutable.
+        {'scheme':'nonempty'}] # Scheme, no netloc -> isabsolute is mutable.
+      for kwargs in mutable:
+         f = furl.furl().set(path=path, **kwargs)
+         if path and path.startswith('/'):
+           assert f.path.isabsolute
+         else:
+           assert not f.path.isabsolute
+         f.path.isabsolute = False # No exception.
+         assert not f.path.isabsolute and not str(f.path).startswith('/')
+         f.path.isabsolute = True # No exception.
+         assert f.path.isabsolute and str(f.path).startswith('/')
 
-      p = furl.Path(absolute_if_not_empty=False)
-      p.set(path)
-      if path and path[0] == '/':
-        assert p.isabsolute
+      # A URL path's isabsolute attribute is read-only if there's a netloc.
+      readonly = [
+        # Netloc, no scheme -> isabsolute is read-only if path is non-empty.
+        {'netloc':'nonempty'},
+        # Netloc and scheme -> isabsolute is read-only if path is non-empty.
+        {'scheme':'nonempty', 'netloc':'nonempty'}]
+      for kwargs in readonly:
+        f = furl.furl().set(path=path, **kwargs)
+        if path: # Exception raised.
+          with self.assertRaises(AttributeError):
+            f.path.isabsolute = False
+          with self.assertRaises(AttributeError):
+            f.path.isabsolute = True
+        else: # No exception raised.
+          f.path.isabsolute = False
+          assert not f.path.isabsolute and not str(f.path).startswith('/')
+          f.path.isabsolute = True
+          assert f.path.isabsolute and str(f.path).startswith('/')
+
+      # A Fragment path's isabsolute attribute is never read-only.
+      f = furl.furl().set(fragment_path=path)
+      if path and path.startswith('/'):
+        assert f.fragment.path.isabsolute
       else:
-        assert not p.isabsolute
+        assert not f.fragment.path.isabsolute
+      f.fragment.path.isabsolute = False # No exception.
+      assert (not f.fragment.path.isabsolute and
+              not str(f.fragment.path).startswith('/'))
+      f.fragment.path.isabsolute = True # No exception.
+      assert f.fragment.path.isabsolute and str(f.fragment.path).startswith('/')
+
+      # Sanity checks.
+      f = furl.furl().set(scheme='mailto', path='dad@pumps.biz')
+      assert str(f) == 'mailto:dad@pumps.biz' and not f.path.isabsolute
+      f.path.isabsolute = True # No exception.
+      assert str(f) == 'mailto:/dad@pumps.biz' and f.path.isabsolute
+      
+      f = furl.furl().set(scheme='sup', fragment_path='/dad@pumps.biz')
+      assert str(f) == 'sup:#/dad@pumps.biz' and f.fragment.path.isabsolute
+      f.fragment.path.isabsolute = False # No exception.
+      assert str(f) == 'sup:#dad@pumps.biz' and not f.fragment.path.isabsolute
 
   def test_nonzero(self):
     p = furl.Path()
@@ -367,27 +421,6 @@ class TestPath(unittest.TestCase):
     assert p
 
 
-class TestPathCompositionInterface(unittest.TestCase):
-  def test_interface(self):
-    class tester(furl.PathCompositionInterface):
-      def __init__(self):
-        furl.PathCompositionInterface.__init__(self)
-
-      def __setattr__(self, attr, value):
-        if not furl.PathCompositionInterface.__setattr__(self, attr, value):
-          object.__setattr__(self, attr, value)
-
-    t = tester()
-    assert isinstance(t.path, furl.Path)
-    assert t.pathstr == ''
-
-    t.path = 'pump/dump'
-    assert isinstance(t.path, furl.Path)
-    assert t.pathstr == 'pump/dump'
-    assert t.path.segments == ['pump', 'dump']
-    assert not t.path.isabsolute
-
-
 class TestQuery(unittest.TestCase):
   def setUp(self):
     # All interaction with parameters is unquoted unless that interaction is
@@ -590,11 +623,11 @@ class TestQueryCompositionInterface(unittest.TestCase):
 
     t = tester()
     assert isinstance(t.query, furl.Query)
-    assert t.querystr == ''
+    assert str(t.query) == ''
 
     t.query = 'a=a&s=s s'
     assert isinstance(t.query, furl.Query)
-    assert t.querystr == 'a=a&s=s+s'
+    assert str(t.query) == 'a=a&s=s+s'
     assert t.args == t.query.params == {'a':'a', 's':'s s'}
 
 
@@ -609,7 +642,7 @@ class TestFragment(unittest.TestCase):
     assert str(f) == 'yasup?sup=foo'
     f.path = '/yasup'
     assert str(f) == '/yasup?sup=foo'
-    assert str(f.query) == f.querystr == 'sup=foo'
+    assert str(f.query) == 'sup=foo'
     f.query.params['sup'] = 'kwlpumps'
     assert str(f) == '/yasup?sup=kwlpumps'
     f.query = ''
@@ -749,10 +782,10 @@ class TestFragmentCompositionInterface(unittest.TestCase):
     assert isinstance(t.fragment, furl.Fragment)
     assert isinstance(t.fragment.path, furl.Path)
     assert isinstance(t.fragment.query, furl.Query)
-    assert t.fragmentstr == ''
+    assert str(t.fragment) == ''
     assert t.fragment.separator
-    assert t.fragment.pathstr == ''
-    assert t.fragment.querystr == ''
+    assert str(t.fragment.path) == ''
+    assert str(t.fragment.query) == ''
 
     t.fragment = 'animal meats'
     assert isinstance(t.fragment, furl.Fragment)
@@ -760,10 +793,10 @@ class TestFragmentCompositionInterface(unittest.TestCase):
     t.fragment.query = 'a=a&s=s+s'
     assert isinstance(t.fragment.path, furl.Path)
     assert isinstance(t.fragment.query, furl.Query)
-    assert t.fragment.pathstr == 'pump/dump'
+    assert str(t.fragment.path) == 'pump/dump'
     assert t.fragment.path.segments == ['pump', 'dump']
     assert not t.fragment.path.isabsolute
-    assert t.fragment.querystr == 'a=a&s=s+s'
+    assert str(t.fragment.query) == 'a=a&s=s+s'
     assert t.fragment.args == t.fragment.query.params == {'a':'a', 's':'s s'}
 
 
@@ -892,10 +925,10 @@ class TestFurl(unittest.TestCase):
     assert f.netloc == 'www.pumps.com'
     assert f.host == 'www.pumps.com'
     assert f.port == 80
-    assert str(f.path) == f.pathstr == '/'
-    assert str(f.query) == f.querystr  == ''
+    assert str(f.path) == '/'
+    assert str(f.query) == ''
     assert f.args == f.query.params == {}
-    assert str(f.fragment) == f.fragmentstr == ''
+    assert str(f.fragment) == ''
     assert f.url == str(f) == url.lower()
     assert f.url == furl.furl(f).url == furl.furl(f.url).url
     assert f is not f.copy() and f.url == f.copy().url
@@ -906,10 +939,10 @@ class TestFurl(unittest.TestCase):
     assert f.netloc == 'www.yahoo.co.uk'
     assert f.host == 'www.yahoo.co.uk'
     assert f.port == 443
-    assert f.pathstr == str(f.path) == '/one/two/three'
-    assert f.querystr == str(f.query) == 'a=a&b=b&m=m%26m'
+    assert str(f.path) == '/one/two/three'
+    assert str(f.query) == 'a=a&b=b&m=m%26m'
     assert f.args == f.query.params == {'a':'a', 'b':'b', 'm':'m&m'}
-    assert str(f.fragment) == f.fragmentstr == 'fragment'
+    assert str(f.fragment) == 'fragment'
     assert f.url == str(f) == url.lower()
     assert f.url == furl.furl(f).url == furl.furl(f.url).url
     assert f is not f.copy() and f.url == f.copy().url
@@ -920,21 +953,21 @@ class TestFurl(unittest.TestCase):
     assert f.netloc == '192.168.1.102:8080'
     assert f.host == '192.168.1.102'
     assert f.port == 8080
-    assert f.pathstr == str(f.path) == '///one//a%20b////'
-    assert f.querystr == str(f.query) == 's=kwl+string'
+    assert str(f.path) == '///one//a%20b////'
+    assert str(f.query) == 's=kwl+string'
     assert f.args == f.query.params == {'s':'kwl string'}
-    assert str(f.fragment) == f.fragmentstr == 'frag'
+    assert str(f.fragment) == 'frag'
     query_quoted = 'sup://192.168.1.102:8080///one//a%20b////?s=kwl+string#frag'
     assert f.url == str(f) == query_quoted
     assert f.url == furl.furl(f).url == furl.furl(f.url).url
     assert f is not f.copy() and f.url == f.copy().url
 
-    # URL paths are always absolute if not empty.
+    # URL paths are optionally absolute if scheme and netloc are empty.
     f = furl.furl()
     f.path.segments = ['pumps']
-    assert str(f.path) == '/pumps'
+    assert str(f.path) == 'pumps'
     f.path = 'pumps'
-    assert str(f.path) == '/pumps'
+    assert str(f.path) == 'pumps'
 
     # Fragment paths are optionally absolute, and not absolute by default.
     f = furl.furl()
@@ -995,15 +1028,15 @@ class TestFurl(unittest.TestCase):
     f = furl.furl('')
     assert f.username is f.password is None
     assert f.scheme is f.host is f.port is f.netloc is None
-    assert f.pathstr == str(f.path) == ''
-    assert f.querystr == str(f.query) == ''
+    assert str(f.path) == ''
+    assert str(f.query) == ''
     assert f.args == f.query.params == {}
-    assert str(f.fragment) == f.fragmentstr == ''
+    assert str(f.fragment) == ''
     assert f.url == ''
 
     # Keep in mind that ';' is a query delimeter for both the URL query and the
-    # fragment query, resulting in the pathstr, querystr, and fragmentstr values
-    # below.
+    # fragment query, resulting in the str(path), str(query), and str(fragment)
+    # values below.
     url = ("pron://example.com/:@-._~!$&'()*+,=;:@-._~!$&'()*+,=:@-._~!$&'()*+,"
            "==?/?:@-._~!$'()*+,;=/?:@-._~!$'()*+,;==#/?:@-._~!$&'()*+,;=")
     pathstr = "/:@-._~!$&'()*+,=;:@-._~!$&'()*+,=:@-._~!$&'()*+,=="
@@ -1014,18 +1047,18 @@ class TestFurl(unittest.TestCase):
     assert f.host == 'example.com'
     assert f.port == None
     assert f.netloc == 'example.com'
-    assert f.pathstr == str(f.path) == pathstr
-    assert f.querystr == str(f.query) == querystr
-    assert f.fragmentstr == str(f.fragment) == fragmentstr
+    assert str(f.path) == pathstr
+    assert str(f.query) == querystr
+    assert str(f.fragment) == fragmentstr
 
     # Scheme only.
     f = furl.furl('sup://')
     assert f.scheme == 'sup'
     assert f.host is f.port is f.netloc is None
-    assert f.pathstr == str(f.path) == ''
-    assert f.querystr == str(f.query) == ''
+    assert str(f.path) == ''
+    assert str(f.query) == ''
     assert f.args == f.query.params == {}
-    assert str(f.fragment) == f.fragmentstr == ''
+    assert str(f.fragment) == ''
     assert f.url == 'sup://' and f.netloc is None
     f.scheme = None
     assert f.scheme is None and f.netloc is None and f.url == ''
@@ -1047,6 +1080,11 @@ class TestFurl(unittest.TestCase):
     f.port = None
     assert f.url == '' and f.netloc is None
 
+    # urlparse.urlsplit() treats the first two '//' as the beginning of a
+    # netloc, even if the netloc is empty.
+    f = furl.furl('////path')
+    assert f.url == '//path' and str(f.path) == '//path'
+
     # TODO(grun): Test more odd urls.
 
   def test_hosts(self):
@@ -1156,16 +1194,16 @@ class TestFurl(unittest.TestCase):
                       fragment_path='1', fragment_args={'f':'frp'})
     assert self._param(f.url, 'a', 'a')
     assert self._param(f.url, 'm', 'm&m')
-    assert f.fragmentstr == str(f.fragment) == '1?f=frp'
-    assert f.pathstr == urlparse.urlsplit(f.url).path == '/kwl%20jump'
+    assert str(f.fragment) == '1?f=frp'
+    assert str(f.path) == urlparse.urlsplit(f.url).path == '/kwl%20jump'
 
     assert f is f.add(path='dir', fragment_path='23', args={'b':'b'},
                       fragment_args={'b':'bewp'})
     assert self._param(f.url, 'a', 'a')
     assert self._param(f.url, 'm', 'm&m')
     assert self._param(f.url, 'b', 'b')
-    assert f.pathstr == str(f.path) == '/kwl%20jump/dir'
-    assert str(f.fragment) == f.fragmentstr == '1/23?f=frp&b=bewp'
+    assert str(f.path) == '/kwl%20jump/dir'
+    assert str(f.fragment) == '1/23?f=frp&b=bewp'
 
     # Supplying both <args> and <query_params> should raise a warning.
     with warnings.catch_warnings(True) as w1:
@@ -1186,7 +1224,7 @@ class TestFurl(unittest.TestCase):
     # <params>.
     assert f is f.set(args={'a':'a a'}, path='path path/dir', port='999',
                       fragment='moresup', scheme='sup', host='host')
-    assert f.pathstr == '/path%20path/dir'
+    assert str(f.path) == '/path%20path/dir'
     assert f.url == 'sup://host:999/path%20path/dir?a=a+a#moresup'
 
     # Path as a list of path segments to join.
@@ -1295,8 +1333,8 @@ class TestFurl(unittest.TestCase):
     assert self._param(f.url, 'a', 'a')
     assert self._param(f.url, 'b', 'b')
     assert self._param(f.url, 's', 's s')
-    assert f.pathstr == '/a/big/path/'
-    assert f.fragment.pathstr == 'a%20frag'
+    assert str(f.path) == '/a/big/path/'
+    assert str(f.fragment.path) == 'a%20frag'
     assert f.fragment.args == {'a':'a', 'with':'args'}
     
     # Path as a list of paths to join before removing.
@@ -1349,6 +1387,12 @@ class TestFurl(unittest.TestCase):
     for join, result in run_tests:
       assert f is f.join(join) and f.url == result
 
+    # Join other furl object, which serialize to strings with str().
+    f = furl.furl('')
+    for join, result in run_tests:
+      tojoin = furl.furl(join)
+      assert f is f.join(tojoin) and f.url == result
+
   def test_equality(self):
     assert furl.furl() is not furl.furl() and furl.furl() == furl.furl()
 
